#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-09-19 16:24
# @Author  : Jieay
# @File    : user_key.py

from __future__ import absolute_import, unicode_literals
from django.core.management import BaseCommand
from utils.user.membersapi import set_aksk_to_db, up_aksk_to_db


class Command(BaseCommand):
    help = """
    python manage.py user_key
    """

    def add_arguments(self, parser):
        parser.add_argument(
            '--add',
            type=str,
            default=None,
            help='创建用户秘钥，用户名',
        )
        parser.add_argument(
            '--update',
            type=str,
            default=None,
            help='修改用户秘钥，用户名',
        )

    def handle(self, *args, **options):
        user_key_add = options.get("add")
        user_key_update = options.get("update")
        key_data = {}
        if user_key_add is not None:
            key_data = set_aksk_to_db(user_key_add)
        elif user_key_update is not None:
            key_data = up_aksk_to_db(user_key_update)
        else:
            print('请输入的用户名。')
        if key_data:
            print('access_key：{access_key} \nsecret_key：{secret_key}'.format(**key_data))
        else:
            print('输入的用户名不正确。')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:18
# @Author  : Jieay
# @File    : api_view_external.py
from rest_framework import status as rest_status
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.authentication import SessionAuthentication, BasicAuthentication, TokenAuthentication
from rest_framework.permissions import IsAuthenticated, AllowAny
from django.http import JsonResponse
from django.db.models import Q
from utils.comm.commapi import set_base_response_data
from utils.frame.pagination import CustomBasePagination


code_msg_map = {
    0: '',
    301: '资源地址重定向',
    400: '参数错误',
    401: '用户校验失败',
    403: '访问未授权资源',
    404: '访问资源不存在',
    405: '访问method错误',
    406: '拒绝接受此请求',
    429: '请求过于频繁，请过会儿来试试~',
    500: '服务器迷路了。',
    507: '数据异常'
}

# 自定义错误码
custom_error_codes = [401, 405, 406, 429]
http_error_codes = [400]


class BaseApiView(APIView):
    """对外接口基类"""
    @classmethod
    def as_v1_view(cls):
        v3_view = cls.as_view()
        
        def view(request, *args, **kwargs):
            response = v3_view(request, *args, **kwargs)
            if isinstance(response, BaseResponse):
                code = response.data.get("code", 0)
                msg = response.data.get("msg", "")
                data = response.data.get("data", {})
                status = 200
                if code in custom_error_codes:
                    status = code
                    data = {
                        "msg": msg,
                        "err_code": code,
                        "data": data,
                    }
                if code in http_error_codes:
                    status = code
                    data = {
                        "msg": msg,
                        "err_code": code,
                        "data": data,
                    }
                return JsonResponse(data=data, status=status, safe=False)
            else:
                return response
        if hasattr(v3_view, 'csrf_exempt'):
            view.csrf_exempt = getattr(v3_view, 'csrf_exempt')
        return view


class BaseAdminApiView(BaseApiView):
    """后台用的接口基类"""
    authentication_classes = (TokenAuthentication, SessionAuthentication, BasicAuthentication)
    permission_classes = (IsAuthenticated,)


class BaseOpenApiView(BaseApiView):
    authentication_classes = ()
    permission_classes = (AllowAny,)


class BaseResponse(Response):
    def __init__(self, code=0, msg='', data=None, status=False):
        if not isinstance(data, list):
            data = data or {}
        if isinstance(data, dict):
            code = data.get('code', code)
            msg = data.get('msg', msg) or code_msg_map[code] or msg
        else:
            msg = msg or code_msg_map[code]

        if isinstance(data, dict):
            _data = {'status': status, 'code': code, 'msg': msg}
            if data.get('data', None) is None:
                _data['data'] = set_base_response_data(data)
            else:
                _data.update(data)
        else:
            _data = {
                'status': status,
                'code': code,
                'msg': msg,
                'data': data
            }
        super(BaseResponse, self).__init__(data=_data, status=rest_status.HTTP_200_OK)


class CustomBaseLimitOffsetPagination(CustomBasePagination):
    pass


class CustomBaseFilter(object):
    model = None
    # ['field1', 'field2']
    fields = []
    ordering_fields = []

    # {'field1': 'contains', 'field2': 'exact'}
    filter_single = {}
    # {'key_words': ['field1', 'field2']}
    filter_multiple = {}
    queryset = None

    def get_filter_single(self, request):
        check_exacts = {}
        if self.filter_single:
            for sk, sv in self.filter_single.items():
                req_v = request.GET.get(sk)
                ck_fields_name = '{}__{}'.format(sk, sv)
                if req_v:
                    check_exacts[ck_fields_name] = req_v
        return check_exacts

    def get_filter_multiple(self, request):
        mq_keys = ''
        mq_obj = Q()
        mq_obj.connector = 'OR'
        if self.filter_multiple:
            key_fields = []
            for mqk, mqv in self.filter_multiple.items():
                req_mqk = request.GET.get(mqk)
                for i in mqv:
                    # mq_name = '{}__contains="{}"'.format(i, req_mqk)
                    mq_name = '{}__contains'.format(i)
                    if req_mqk:
                        r_mq = Q()
                        r_mq.connector = 'OR'
                        r_mq.children.append((mq_name, req_mqk))
                        mq_obj.add(r_mq, 'OR')
                        key_fields.append(mq_name)
            if key_fields:
                mq_keys = ' | '.join(key_fields)
        if mq_keys:
            return True, mq_obj
        return False, mq_obj

    def get_queryset(self, request):
        check_exacts = self.get_filter_single(request)
        mq_keys, mq_obj = self.get_filter_multiple(request)
        if mq_keys:
            if self.queryset is not None:
                queryset = self.queryset.filter(mq_obj)
            else:
                queryset = self.model.objects.filter(mq_obj)
        else:
            if self.queryset is not None:
                queryset = self.queryset
            else:
                queryset = self.model.objects.all()

        if check_exacts:
            queryset = queryset.filter(**check_exacts)

        queryset = queryset.values(*self.fields)
        queryset = queryset.order_by(*self.ordering_fields)
        return list(queryset)


class CustomBaseAdminApiView(BaseAdminApiView, CustomBaseFilter):

    def get(self, request):
        data_list = self.get_queryset(request)
        page = CustomBaseLimitOffsetPagination()
        data_list_page = page.paginate_queryset(queryset=data_list, request=request, view=self)
        if data_list_page:
            return page.get_paginated_response(data_list_page)
        data = data_list
        return BaseResponse(code=404, data=data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:30
# @Author  : Jieay
# @File    : model_view_set.py
from rest_framework import status
from rest_framework import viewsets
from rest_framework.serializers import Serializer
from rest_framework.response import Response
from rest_framework.authentication import SessionAuthentication, BasicAuthentication, TokenAuthentication
from rest_framework.permissions import IsAuthenticated, AllowAny
from app.models.common.fake_model import FakeModel
from app.serializers.common.fake_serializer import FakeSerializer
from utils.comm.commapi import get_serializer_data_to_json, clean_response_field
from utils.frame.pagination import RbacPageNumberPagination


import logging
logger = logging.getLogger(__name__)


class CustomBaseSessionAuthentication(SessionAuthentication):
    """
    Use Django's session framework for authentication.
    """

    def authenticate(self, request):
        """
        Returns a `User` if the request session currently has a logged in user.
        Otherwise returns `None`.
        """

        # Get the session-based user from the underlying HttpRequest object
        user = getattr(request._request, 'user', None)

        # Unauthenticated, CSRF validation not required
        if not user or not user.is_active:
            return None

        self.enforce_csrf(request)

        # CSRF passed with authenticated user
        return user, None


class BaseModelViewSet(viewsets.ModelViewSet):
    """ModelViewSet认证基类"""
    authentication_classes = (TokenAuthentication, CustomBaseSessionAuthentication, BasicAuthentication)
    permission_classes = (IsAuthenticated,)


class BaseReadOnlyModelViewSet(viewsets.ReadOnlyModelViewSet):
    """ModelViewSet只读认证基类"""
    authentication_classes = (TokenAuthentication, CustomBaseSessionAuthentication, BasicAuthentication)
    permission_classes = (IsAuthenticated,)


class BaseOPenReadOnlyModelViewSet(viewsets.ReadOnlyModelViewSet):
    """ModelViewSet只读开放基类"""
    authentication_classes = ()
    permission_classes = (AllowAny,)


class CustomBaseJsonResponse(Response):
    """ModelViewSet接口返回基类"""

    def __init__(self, data=None, code=None, msg=None, status=None,
                 template_name=None, headers=None,
                 exception=False, content_type=None):
        """
        Alters the init arguments slightly.
        For example, drop 'template_name', and instead use 'data'.
        Setting 'renderer' and 'media_type' will typically be deferred,
        For example being set automatically by the `APIView`.
        """
        super(Response, self).__init__(None, status=code)

        if isinstance(data, Serializer):
            msg = (
                'You passed a Serializer instance as data, but '
                'probably meant to pass serialized `.data` or '
                '`.error`. representation.'
            )
            raise AssertionError(msg)

        self.data = {"status": status, "code": code, "msg": msg, "data": data}
        self.template_name = template_name
        self.exception = exception
        self.content_type = content_type

        if headers:
            for name, value in headers.iteritems(headers):
                self[name] = value


class CustomBaseViewSet(BaseModelViewSet):
    """自定义ModelViewSet认证接口基类"""

    queryset = FakeModel.objects.all()
    serializer_class = FakeSerializer
    del_fields = ['soft_del']

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            data = get_serializer_data_to_json(serializer.data)
            data = clean_response_field(data, self.del_fields)
            return self.get_paginated_response(data)

        serializer = self.get_serializer(queryset, many=True)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data)

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        logger.info('Create: {}'.format(serializer.data))
        data = get_serializer_data_to_json(serializer.data)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_201_CREATED,
                                      headers=headers,
                                      msg="Created successfully.",
                                      status=True)

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        logger.info('Update: {}'.format(serializer.data))
        data = get_serializer_data_to_json(serializer.data)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Updated successfully.",
                                      status=True)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Queried successfully.",
                                      status=True)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        _id = instance.id
        self.perform_destroy(instance)
        logger.info('Delete: {}'.format([{"id": _id}]))
        return CustomBaseJsonResponse(data=[{"id": _id}],
                                      code=status.HTTP_200_OK,
                                      msg="Deleted successfully.",
                                      status=True)


class CustomBaseReadOnlyViewSet(BaseReadOnlyModelViewSet):
    """自定义ModelViewSet只读认证接口基类"""

    queryset = FakeModel.objects.all()
    serializer_class = FakeSerializer
    del_fields = ['soft_del']

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            data = get_serializer_data_to_json(serializer.data)
            data = clean_response_field(data, self.del_fields)
            return self.get_paginated_response(data)

        serializer = self.get_serializer(queryset, many=True)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Queried successfully.",
                                      status=True)


class CustomOPenBaseViewSet(CustomBaseViewSet):
    """自定义ModelViewSet开放接口基类"""
    authentication_classes = ()
    permission_classes = (AllowAny,)


class CustomBaseOpenReadOnlyViewSet(BaseOPenReadOnlyModelViewSet):
    """自定义ModelViewSet开放接口基类"""

    queryset = FakeModel.objects.all()
    serializer_class = FakeSerializer
    del_fields = ['soft_del']

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            data = get_serializer_data_to_json(serializer.data)
            data = clean_response_field(data, self.del_fields)
            return self.get_paginated_response(data)

        serializer = self.get_serializer(queryset, many=True)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Queried successfully.",
                                      status=True)


class RbacOpenReadOnlyViewSet(BaseOPenReadOnlyModelViewSet):
    queryset = FakeModel.objects.all()
    serializer_class = FakeSerializer
    del_fields = ['soft_del']
    pagination_class = RbacPageNumberPagination

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            data = get_serializer_data_to_json(serializer.data)
            data = clean_response_field(data, self.del_fields)
            return self.get_paginated_response(data)

        serializer = self.get_serializer(queryset, many=True)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data)

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Queried successfully.",
                                      status=True)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-03 10:29
# @Author  : Jieay
# @File    : membersapi.py

from rest_framework.authentication import get_authorization_header
from rest_framework.authtoken.models import Token
from app.models.user.members import Members
from utils.comm.commapi import get_random


def get_aksk():
    access_key = get_random(17, ty='tystr')
    secret_key = get_random(25, ty='tystr')
    return access_key, secret_key


def set_aksk_to_db(username):
    access_key, secret_key = get_aksk()
    user_obj = Members.objects.filter(username=username)
    data = {}
    if user_obj.exists():
        if user_obj[0].access_key and user_obj[0].secret_key:
            access_key = user_obj[0].access_key
            secret_key = user_obj[0].secret_key
        else:
            user_obj.update(access_key=access_key, secret_key=secret_key)
        data['access_key'] = access_key
        data['secret_key'] = secret_key
    return data


def up_aksk_to_db(username):
    access_key, secret_key = get_aksk()
    user_obj = Members.objects.filter(username=username)
    data = {}
    if user_obj.exists():
        user_obj.update(access_key=access_key, secret_key=secret_key)
        data['access_key'] = access_key
        data['secret_key'] = secret_key
    return data


def get_request_token(request):
    auth = get_authorization_header(request).split()
    if len(auth) == 2:
        key = auth[1].decode()
    else:
        key = None
    token = Token.objects.select_related('user').get(key=key)
    return token


class GetUserInfo(object):

    def __init__(self, request):
        self.request = request

    def get_token(self):
        return get_request_token(self.request)

    def get_user_info(self):
        token = self.get_token()
        if token is not None:
            return token.user
        return

    def user_id(self):
        user_info = self.get_user_info()
        if user_info is not None:
            return user_info.id
        return

    def user_name(self):
        _name = None
        user_info = self.get_user_info()
        if user_info is not None:
            if user_info.name:
                _name = user_info.name
            elif user_info.username:
                _name = user_info.username
        return _name

    def user_oaid(self):
        user_info = self.get_user_info()
        if user_info is not None:
            return user_info.oaid
        return


def get_user_id(request):
    user_info = GetUserInfo(request)
    return user_info.user_id()


def get_user_name(request):
    user_info = GetUserInfo(request)
    return user_info.user_name()


def get_user_oaid(request):
    user_info = GetUserInfo(request)
    return user_info.user_oaid()


def get_aksk_token(access_key, secret_key):
    token = None
    if access_key and secret_key:
        user_obj = Members.objects.filter(access_key=access_key, secret_key=secret_key)
        if user_obj.exists():
            user = user_obj[0]
            token_obj = Token.objects.filter(user=user)
            if token_obj.exists():
                token = token_obj[0].key
            else:
                add_token = Token.objects.update_or_create(user=user)
                token = add_token[0].key
    return token


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:25
# @Author  : Jieay
# @File    : throttle.py
from rest_framework.throttling import ScopedRateThrottle


class CustomBaseThrottle(ScopedRateThrottle):

    def get_cache_key(self, request, view):
        """
        Should return a unique cache-key which can be used for throttling.
        Must be overridden.

        May return `None` if the request should not be throttled.
        """
        if not request.user:
            ident = self.get_ident(request)
        else:
            ident = request.user

        return self.cache_format % {
            'scope': self.scope,
            'ident': ident
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from django.conf import settings
import redis

RDHSOT = settings.REDIS_HOST
RDPASSWORD = settings.REDIS_PASSWORD
RDPORT = settings.REDIS_PORT
RDDB = settings.REDIS_BASE_DB


class MyRedis(object):

    def __init__(self, **redisinfo):
        """
        :param redisinfo: host, port, db, password
        """
        self.host = redisinfo.get('host', RDHSOT)
        self.port = redisinfo.get('port', RDPORT)
        self.db = redisinfo.get('db', RDDB)
        self.passwd = redisinfo.get('password', RDPASSWORD)
        redis_info = {
            "host": self.host,
            "port": self.port,
            "db": self.db
        }
        if self.passwd:
            redis_info['password'] = self.passwd
        redis_poolinfo = redis.ConnectionPool(**redis_info)
        self.__r = redis.Redis(connection_pool=redis_poolinfo)

    def set(self, key, value, timeout):
        self.__r.set(key, value)
        self.__r.expire(key, timeout)
        
    def get(self, key):
        return self.__r.get(key)
        
    def exists(self, key):
        return self.__r.exists(key)
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/20 4:26 下午
# @Author  : Jieay
# @File    : __init__.py.py

from app.models.common.fake_model import *
from app.models.user.members import *


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-11-15 11:39
# @Author  : Jieay
# @File    : health_view.py

from drf_yasg.utils import swagger_auto_schema
from drf_yasg import openapi
from utils.frame.api_view_external import BaseOpenApiView, BaseResponse
from utils.frame.pagination import DrfYasgDjangoRestResponsePagination

import logging
logger = logging.getLogger(__name__)


class GetHealthCheckView(BaseOpenApiView):

    data_response = openapi.Response('健康检查', openapi.Schema('check', type=openapi.TYPE_STRING))

    @swagger_auto_schema(operation_description="健康检查接口",
                         responses={200: data_response, 404: 'Data not found.'},
                         paginator_inspectors=[DrfYasgDjangoRestResponsePagination])
    def get(self, request):
        data = {"check": 'ok'}
        return BaseResponse(code=200, data=data, status=True, msg='Queried successfully.')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/20 4:28 下午
# @Author  : Jieay
# @File    : urls.py

from django.urls import re_path, include
from rest_framework import routers


urlpatterns = []

router = routers.DefaultRouter()

from app.views.user.members_view import MembersViewsSet, MembersView

# 使用 ModelViewSet 方式写接口，通过 router 方式注册 url
# 使用时需要注意，需要以/结尾，例如：/api/v1/users/
router.register(r'users', MembersViewsSet)


# 使用 APIView 方式写接口，用过直接添加 urlpatterns 注册 url
urlpatterns += [
    re_path(r'^v1/members/list/$', MembersView.as_view()),
]


urlpatterns += [
    re_path(r'v1/', include(router.urls)),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/1/20 4:32 下午
# @Author  : Jieay
# @File    : members.py
import json
from django.db import models
from django.contrib.auth.models import AbstractUser
from utils.comm.commapi import check_json_format


class UserDepartmentsModel(models.Model):
    id = models.IntegerField(primary_key=True, unique=True, verbose_name='ID')
    name = models.CharField(max_length=64, blank=True, null=True, verbose_name='部门名称')
    parentid = models.IntegerField(blank=True, null=True, verbose_name='上级部门ID')
    order = models.IntegerField(blank=True, null=True, verbose_name='排序ID')
    desc = models.TextField(blank=True, null=True, verbose_name='描述')
    comments = models.TextField(blank=True, null=True, verbose_name='备注')
    ctime = models.DateTimeField(auto_now_add=True, verbose_name='创建时间')
    mtime = models.DateTimeField(auto_now=True, verbose_name='修改时间')

    def __str__(self):
        return self.name

    class Meta:
        db_table = "app_user_department"
        verbose_name_plural = "部门信息"


class Members(AbstractUser):
    name = models.CharField(max_length=32, blank=True, null=True, verbose_name="姓名", help_text="中文用户名称")
    unit_name = models.CharField(max_length=64, blank=True, null=True, verbose_name="公司名称")
    dept_name = models.CharField(max_length=64, blank=True, null=True, verbose_name="部门名称")
    department = models.TextField(
        default=[], blank=True, null=True,
        verbose_name='部门列表', help_text="填写的是部门ID列表，格式是JSON字符串，例如：[1,2]"
    )
    employee_code = models.CharField(max_length=32, blank=True, null=True, verbose_name="员工编号")
    employee_name = models.CharField(max_length=32, blank=True, null=True, verbose_name="员工姓名")
    oaid = models.CharField(max_length=32, blank=True, null=True, verbose_name="OAID", help_text="公司OAID，唯一标识")
    post_name = models.CharField(max_length=64, blank=True, null=True, verbose_name="职位")
    telephone = models.CharField(max_length=11, blank=True, null=True, verbose_name="手机", help_text="11位数字")
    access_key = models.CharField(max_length=64, blank=True, null=True, verbose_name="access_key", help_text="秘钥 AK")
    secret_key = models.CharField(max_length=64, blank=True, null=True, verbose_name="secret_key",  help_text="秘钥 SK")
    ctime = models.DateTimeField(auto_now_add=True, verbose_name='创建时间')
    mtime = models.DateTimeField(auto_now=True, verbose_name='修改时间')

    def __str__(self):
        return self.username

    @property
    def department_info(self):
        data_info = []
        if check_json_format(self.department):
            department = json.loads(self.department)
            for dep in department:
                dep_obj = UserDepartmentsModel.objects.filter(id=dep)
                if dep_obj.exists():
                    data = {'id': dep, 'name': dep_obj[0].name}
                    data_info.append(data)
        return data_info

    class Meta:
        db_table = "app_user_members"
        verbose_name_plural = "用户信息"


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-09-23 17:23
# @Author  : Jieay
# @File    : demo_test.py
from frame_server.celery import celery_app as app
from celery import shared_task
from celery.schedules import crontab
import time


@app.task
def add(x, y):  # yui = add.delay(2,8)
    time.sleep(15)
    return x + y


@app.task()
def mul(x, y):
    time.sleep(20)
    return x * y


@app.task
def xsum(numbers):
    return sum(numbers)


@shared_task()
def timed_test():
    msg = '{}'.format('lai a ce shi a')
    return msg


@app.task(queue='democi', name='app.tasks.demo_test.demo_ci_queue_test')
def demo_ci_queue_test(x, y):
    time.sleep(20)
    return x * y



# -*- coding: utf-8 -*-
# @Time    : 2022/9/17 15:47
# @Author  : Jieay
# @File    : main.py
import os
import shutil
import sys
import uuid
from pathlib import Path
from . import __version__

BASE_DIR = Path(__file__).resolve().parent


class ManagementUtility:
    """
    Encapsulate the logic of the pdf-admin utilities.
    """
    VERSION_LIST = ['3.2.13', '2.2.2']

    def __init__(self, argv=None):
        self.argv = argv or sys.argv[:]
        self.prog_name = os.path.basename(self.argv[0])
        self.version_list = self.VERSION_LIST
        self.cwd_dir = os.getcwd()
        self.data_root_dir = os.path.join(BASE_DIR, 'data')
        self.frame_root_dir = os.path.join(BASE_DIR, 'django')
        self.frame_name = 'frame_server'

    def main_help_text(self):
        """Return the script's main help text, as a string."""
        label_name = '\n'.join(['    {}'.format(x) for x in self.version_list])
        usage = [
            "",
            "usage: edf-admin label project_name",
            "labels: ",
            label_name,
        ]
        return '\n'.join(usage)

    def replace_content_read(self, file_name):
        """
        替换内容，读文件内容
        :param file_name: 文件路径
        :return: 文件内容
        """
        with open(file_name, 'r') as f:
            new_content = f.read()
        return new_content

    def replace_content_write(self, file_name, content):
        """
        替换内容，写文件
        :param file_name: 文件路径
        :param content: 文件内容
        :return: 文件路径
        """
        with open(file_name, 'w') as f:
            f.write(content)
        return file_name

    def change_rename(self, old_data, new_data):
        """
        文件或者目录重命名
        :param old_data: 旧未文件或者目录
        :param new_data: 新文件或者目录
        :return: `bool`
        """
        try:
            os.rename(old_data, new_data)
            return True
        except FileNotFoundError:
            return False

    def check_file_exists(self, file_path):
        """
        检查目标是否是文件
        :param file_path: 文件路径
        :return: `bool`
        """
        if os.path.isfile(file_path):
            return True
        return False

    def change_file_to_project_name(self, file_name, project_name):
        """
        更改脚手架项目名称
        :param file_name: 文件路径
        :param project_name: 项目名
        """
        name = os.path.basename(file_name)
        file_path = os.path.dirname(file_name)

        # 将 gitignore 文件重命名
        if name == 'gitignore':
            self.change_rename(file_name, os.path.join(file_path, '.gitignore'))
        else:
            old_content = self.replace_content_read(file_name)
            new_name = uuid.uuid4().hex
            new_file_name = os.path.join(file_path, new_name)
            new_content = old_content.replace(self.frame_name, project_name)
            self.replace_content_write(new_file_name, new_content)
            if os.path.exists(file_name):
                os.remove(file_name)
            # 将修改内容文件重命名原来的文件
            self.change_rename(new_file_name, file_name)

    def change_frame_project_name(self, project_name, cf_path_list):
        """
        修改脚手架项目名称
        :param project_name: 项目名
        :param cf_path_list: 修改文件路径列表
        """
        for cf_file in cf_path_list:
            self.change_file_to_project_name(file_name=cf_file, project_name=project_name)

    def _check_file_path_list(self, project_name, tmp_data, change_file_list):
        """
        检查文件路径列表
        :param project_name: 项目名
        :param tmp_data: 临时目录路径
        :param change_file_list: 修改文件列表
        :return: `list` ['/xxx/xxx/xxx.py', '/aaa/aaa/aaa.py']
        """
        cf_path_list = []
        for name in change_file_list:
            if isinstance(name, list):
                for sub_name in name:
                    old_project_path = os.path.join(tmp_data, self.frame_name)
                    project_path = os.path.join(tmp_data, project_name)
                    if not os.path.exists(project_path) and os.path.exists(old_project_path):
                        self.change_rename(old_project_path, project_path)
                    file_path = os.path.join(project_path, sub_name)
                    if self.check_file_exists(file_path):
                        cf_path_list.append(file_path)
            else:
                if '/' in name:
                    name_list = name.split('/')
                    file_path = os.path.join(tmp_data, *name_list)
                else:
                    file_path = os.path.join(tmp_data, name)
                if self.check_file_exists(file_path):
                    cf_path_list.append(file_path)
        return cf_path_list

    def change_222(self, project_name, tmp_data):
        """修改Django 2.2.2 版本脚手架文件"""
        change_file_list = [
            'config_dev', 'config_dist', 'manage.py', 'README.md', 'gitignore',
            [
                'celery.py', 'settings.py', 'wsgi.py', 'urls.py'
            ],
            'app/tasks/demo_test.py',
        ]
        cf_path_list = self._check_file_path_list(
            project_name=project_name, tmp_data=tmp_data, change_file_list=change_file_list
        )
        self.change_frame_project_name(project_name=project_name, cf_path_list=cf_path_list)

    def change_3213(self, project_name, tmp_data):
        """修改Django 3.2.13 版本脚手架文件"""
        change_file_list = [
            'config_dev', 'config_dist', 'manage.py', 'README.md', 'gitignore',
            [
                'celery.py', 'settings.py', 'wsgi.py', 'urls.py', 'asgi.py'
            ],
            'app/tasks/demo_test.py', 'supervisord.conf', 'uwsgi_config.ini',
            'celery_beat.sh', 'celery_worker.sh',
        ]
        cf_path_list = self._check_file_path_list(
            project_name=project_name, tmp_data=tmp_data, change_file_list=change_file_list
        )
        self.change_frame_project_name(project_name=project_name, cf_path_list=cf_path_list)

    def change_frame_file(self, label, project_name, tmp_data):
        """修改不同Django版本脚手架文件"""
        if label == '2.2.2':
            self.change_222(project_name, tmp_data)

        elif label == '3.2.13':
            self.change_3213(project_name, tmp_data)

    def create_frame(self, label, project_name):
        """
        创建脚手架项目文件
        :param label: Django版本
        :param project_name: 项目名称
        """
        if not os.path.exists(self.data_root_dir):
            os.makedirs(self.data_root_dir)
        tmp_data_dir = os.path.join(self.data_root_dir, uuid.uuid4().hex)
        frame_source_dir = os.path.join(self.frame_root_dir, label)
        # 创建临时文件
        if not os.path.exists(tmp_data_dir):
            shutil.copytree(frame_source_dir, tmp_data_dir)

        self.change_frame_file(label=label, project_name=project_name, tmp_data=tmp_data_dir)

        # 创建项目目录，生产脚手架稳文件
        create_project_dir = os.path.join(self.cwd_dir, project_name)
        if not os.path.exists(create_project_dir):
            shutil.copytree(tmp_data_dir, create_project_dir)

        # 删除临时文件
        if os.path.exists(tmp_data_dir):
            shutil.rmtree(tmp_data_dir)

        sys.stdout.write('Create finished.' + '\n')

    def execute(self):
        """
        Given the command-line arguments, figure out which subcommand is being
        run, create a parser appropriate to that command, and run it.
        """
        sys.stdout.write('Welcome to PDF tools.' + '\n')
        try:
            subcommand = self.argv[1]
        except IndexError:
            subcommand = 'help'  # Display help if no arguments were given.

        if subcommand in self.version_list:
            sys.stdout.write('Select the django version: {}'.format(subcommand) + '\n')
            try:
                project_name = self.argv[2]
            except IndexError:
                project_name = None

            if project_name:
                if '-' in project_name:
                    sys.stdout.write('The project_name【{}】cannot use the - character.'.format(project_name) + '\n')
                    exit(0)
                sys.stdout.write('Create project name: {}'.format(project_name) + '\n')
                self.create_frame(label=subcommand, project_name=project_name)
            else:
                sys.stdout.write(self.main_help_text() + '\n')

        elif subcommand == 'help':
            sys.stdout.write(self.main_help_text() + '\n')

        # Special-cases: We want 'pdf-admin --version' and
        # 'pdf-admin --help' to work, for backwards compatibility.
        elif subcommand == 'version' or self.argv[1:] == ['--version']:
            sys.stdout.write(__version__ + '\n')

        elif self.argv[1:] in (['--help'], ['-h']):
            sys.stdout.write(self.main_help_text() + '\n')
        else:
            sys.stdout.write(self.main_help_text() + '\n')


def execute_from_command_line(argv=None):
    """Run a ManagementUtility."""
    utility = ManagementUtility(argv)
    utility.execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:19
# @Author  : Jieay
# @File    : pagination.py
from rest_framework.pagination import LimitOffsetPagination, PageNumberPagination, CursorPagination
from collections import OrderedDict
from rest_framework.response import Response
from drf_yasg.inspectors import DjangoRestResponsePagination
from drf_yasg import openapi


class DrfYasgDjangoRestResponsePagination(DjangoRestResponsePagination):
    """Provides response schema pagination warpping for django-rest-framework's LimitOffsetPagination,
    PageNumberPagination and CursorPagination
    """

    def get_paginated_response(self, paginator, response_schema):
        assert response_schema.type == openapi.TYPE_ARRAY, "array return expected for paged response"
        paged_schema = None
        if isinstance(paginator, (LimitOffsetPagination, PageNumberPagination, CursorPagination)):
            has_count = not isinstance(paginator, CursorPagination)
            paged_schema = openapi.Schema(
                type=openapi.TYPE_OBJECT,
                properties=OrderedDict((
                    ('status', openapi.Schema(type=openapi.TYPE_BOOLEAN)),
                    ('code', openapi.Schema(type=openapi.TYPE_NUMBER)),
                    ('msg', openapi.Schema(type=openapi.TYPE_STRING)),
                    ('count', openapi.Schema(type=openapi.TYPE_INTEGER) if has_count else None),
                    ('next', openapi.Schema(type=openapi.TYPE_STRING, format=openapi.FORMAT_URI, x_nullable=True)),
                    ('previous', openapi.Schema(type=openapi.TYPE_STRING, format=openapi.FORMAT_URI, x_nullable=True)),
                    ('data', response_schema),
                )),
                required=['data']
            )

            if has_count:
                paged_schema.required.insert(0, 'count')

        return paged_schema


class CustomBasePagination(LimitOffsetPagination):
    """自定义偏移量分页器基类"""

    def get_paginated_response(self, data):
        status = True
        code = 200
        msg = 'Queried successfully.'
        if not data:
            # status = False  # 查不到，理论来说，只是结果为空，但不是出错。
            code = 404
            msg = "Data not found."

        return Response(OrderedDict([
            ('status', status),
            ('code', code),
            ('msg', msg),
            ('count', self.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('data', data),
        ]))


class CustomBasePageNumberPagination(PageNumberPagination):
    """自定义页码分页器基类"""

    # 默认显示每页的数据条数限制
    page_size = 10
    # 每页最大显示数据条数
    max_page_size = 200
    # 页码请求传参参数字段名
    page_query_param = 'number'
    # 每页数据限制请求传参参数字段名
    page_size_query_param = 'size'

    def get_paginated_response(self, data):
        status = True
        code = 200
        msg = 'Queried successfully.'
        if not data:
            # status = False  # 查不到，理论来说，只是结果为空，但不是出错。
            code = 404
            msg = "Data not found."

        return Response(OrderedDict([
            ('status', status),
            ('code', code),
            ('msg', msg),
            ('count', self.page.paginator.count),
            ('number', self.page.number),
            ('size', len(data)),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('data', data)
        ]))


class RbacPageNumberPagination(CustomBasePageNumberPagination):
    # 每页最大显示数据条数
    max_page_size = 9999

    def get_paginated_response(self, data):
        status = True
        code = 200
        msg = 'Queried successfully.'
        if not data:
            # status = False  # 查不到，理论来说，只是结果为空，但不是出错。
            code = 404
            msg = "Data not found."

        data_info = {'data': OrderedDict([
            ('status', status),
            ('code', code),
            ('msg', msg),
            ('count', self.page.paginator.count),
            ('number', self.page.number),
            ('size', len(data)),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('content', data)
        ])}
        return Response(data_info, content_type="application/json; charset=utf-8")



# Generated by Django 2.2.2 on 2020-11-15 12:01

import django.contrib.auth.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.utils.timezone
import utils.custom_fields.list_field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='FakeModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'db_table': 'app_common_fake',
            },
        ),
        migrations.CreateModel(
            name='UserDepartmentsModel',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, unique=True, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=64, null=True, verbose_name='部门名称')),
                ('parentid', models.IntegerField(blank=True, null=True, verbose_name='上级部门ID')),
                ('order', models.IntegerField(blank=True, null=True, verbose_name='排序ID')),
                ('desc', models.TextField(blank=True, null=True, verbose_name='描述')),
                ('comments', models.TextField(blank=True, null=True, verbose_name='备注')),
                ('ctime', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('mtime', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
            ],
            options={
                'verbose_name_plural': '部门信息',
                'db_table': 'app_user_department',
            },
        ),
        migrations.CreateModel(
            name='Members',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('name', models.CharField(blank=True, help_text='中文用户名称', max_length=32, null=True, verbose_name='姓名')),
                ('unit_name', models.CharField(blank=True, max_length=64, null=True, verbose_name='公司名称')),
                ('dept_name', models.CharField(blank=True, max_length=64, null=True, verbose_name='部门名称')),
                ('department', utils.custom_fields.list_field.ListField(blank=True, default=[], help_text='填写的是部门ID列表，格式是JSON字符串，例如：[1,2]', null=True, verbose_name='部门列表')),
                ('employee_code', models.CharField(blank=True, max_length=32, null=True, verbose_name='员工编号')),
                ('employee_name', models.CharField(blank=True, max_length=32, null=True, verbose_name='员工姓名')),
                ('oaid', models.CharField(blank=True, help_text='公司OAID，唯一标识', max_length=32, null=True, verbose_name='OAID')),
                ('post_name', models.CharField(blank=True, max_length=64, null=True, verbose_name='职位')),
                ('telephone', models.CharField(blank=True, help_text='11位数字', max_length=11, null=True, verbose_name='手机')),
                ('access_key', models.CharField(blank=True, help_text='秘钥 AK', max_length=64, null=True, verbose_name='access_key')),
                ('secret_key', models.CharField(blank=True, help_text='秘钥 SK', max_length=64, null=True, verbose_name='secret_key')),
                ('ctime', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('mtime', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name_plural': '用户信息',
                'db_table': 'app_user_members',
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:42
# @Author  : Jieay
# @File    : exception_handler.py
from collections import OrderedDict
from rest_framework.response import Response
from rest_framework.views import exception_handler

import logging
logger = logging.getLogger(__name__)


def CustomBaseExceptionHandler(exc, context):
    """自定义异常返回"""

    detail_msg = str(exc)
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    response = exception_handler(exc, context)
    if response is None:
        # 自定义数据异常，response 为空的时候返回报错信息
        data = {
            'msg': {
                'detail': detail_msg,
                'error': '数据异常'
            },
            'data': [],
            'code': 507
        }
        data_list = [
            ('status', False),
            ('code', data.get('code')),
            ('msg', data.get('msg')),
            ('data', data.get('data'))
        ]
        logger.info('Custom Exception: {}'.format(data_list))
        return Response(OrderedDict(data_list))

    # Now add the HTTP status code to the response.
    if response is not None:
        logger.info('exception_handler: {}'.format(response))
        exception_error_msg = '{}'.format(response.data)
        logger.info('exception_error: {}'.format(exception_error_msg))
        response.data.clear()
        response.data['status'] = False
        response.data['code'] = response.status_code
        response.data['data'] = []

        http_error_status_code = {
            400: "HTTP_400_BAD_REQUEST",
            401: "HTTP_401_UNAUTHORIZED",
            402: "HTTP_402_PAYMENT_REQUIRED",
            403: "HTTP_403_FORBIDDEN",
            404: "HTTP_404_NOT_FOUND",
            405: "HTTP_405_METHOD_NOT_ALLOWED",
            406: "HTTP_406_NOT_ACCEPTABLE",
            407: "HTTP_407_PROXY_AUTHENTICATION_REQUIRED",
            408: "HTTP_408_REQUEST_TIMEOUT",
            409: "HTTP_409_CONFLICT",
            410: "HTTP_410_GONE",
            411: "HTTP_411_LENGTH_REQUIRED",
            412: "HTTP_412_PRECONDITION_FAILED",
            413: "HTTP_413_REQUEST_ENTITY_TOO_LARGE",
            414: "HTTP_414_REQUEST_URI_TOO_LONG",
            415: "HTTP_415_UNSUPPORTED_MEDIA_TYPE",
            416: "HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE",
            417: "HTTP_417_EXPECTATION_FAILED",
            422: "HTTP_422_UNPROCESSABLE_ENTITY",
            423: "HTTP_423_LOCKED",
            424: "HTTP_424_FAILED_DEPENDENCY",
            428: "HTTP_428_PRECONDITION_REQUIRED",
            429: "HTTP_429_TOO_MANY_REQUESTS",
            431: "HTTP_431_REQUEST_HEADER_FIELDS_TOO_LARGE",
            451: "HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS",
            500: "HTTP_500_INTERNAL_SERVER_ERROR",
            501: "HTTP_501_NOT_IMPLEMENTED",
            502: "HTTP_502_BAD_GATEWAY",
            503: "HTTP_503_SERVICE_UNAVAILABLE",
            504: "HTTP_504_GATEWAY_TIMEOUT",
            505: "HTTP_505_HTTP_VERSION_NOT_SUPPORTED",
            507: "HTTP_507_INSUFFICIENT_STORAGE",
            511: "HTTP_511_NETWORK_AUTHENTICATION_REQUIRED",
        }

        response.data['msg'] = {}
        response.data['msg']['http_status'] = http_error_status_code[response.status_code]
        response.data['msg']['error_msg'] = exception_error_msg

    return Response(OrderedDict([
        ('status', response.data['status']),
        ('code', response.data['code']),
        ('msg', response.data['msg']),
        ('data', response.data['data']),
    ]))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 18:19
# @Author  : Jieay
# @File    : celery.py
import logging
import os

from celery import Celery
from celery import Task
from django.conf import settings


logger = logging.getLogger(__name__)

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'frame_server.settings')


class LogErrorsTask(Task):
    def on_failure(self, exc, task_id, args, kwargs, einfo):
        logging.exception(exc)


celery_app = Celery('frame_server', broker=settings.BROKER_URL, task_cls=LogErrorsTask)

celery_app.config_from_object('django.conf:settings')

celery_app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:42
# @Author  : Jieay
# @File    : custom_filters.py

from django_filters.rest_framework import BaseInFilter, NumberFilter, BaseRangeFilter


class NumberInFilter(BaseInFilter, NumberFilter):
    pass


class NumberRangeFilter(BaseRangeFilter, NumberFilter):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/9/10 2:31 下午
# @Author  : Jieay
# @File    : logapi.py

import logging
import uuid

"""
linux终端下输出带颜色的文字只需在文字前面添加如下格式
\033[显示方式;前景色;背景色m

显示方式	意义
0	终端默认设置
1	高亮显示
4	使用下划线
5	闪烁
7	反白显示
8	不可见

前景色	背景色	颜色
30	40	黑色
31	41	红色
32	42	绿色
33	43	黃色
34	44	蓝色
35	45	紫红色
36	46	青蓝色
37	47	白色
"""
NONE = '\033[0m'
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = ('\033[0;%dm' % i for i in range(30, 38))


class DefaultServerFormatter(logging.Formatter):
    """自定义日志输出格式中间件"""
    COLORS = {
        'CRITICAL': RED,
        'ERROR': RED,
        'WARNING': YELLOW,
        'INFO': GREEN,
        'DEBUG': BLUE,
    }

    def __init__(self, *args, **kwargs):
        super(DefaultServerFormatter, self).__init__(*args, **kwargs)

    def format(self, record):
        # 日志中添加一些自定义请求相关的数据
        try:
            from .GlobalRequestMiddleware import global_request
            record.user = getattr(global_request.user, 'username', '-----------')
            record.ip = getattr(global_request, 'ip', None) or global_request.META.get('REMOTE_ADDR', None)
            record.request_id = global_request.id
        except Exception as ex:
            record.user = 'NotUser'
            record.ip = 'NotIp'
            record.request_id = uuid.uuid4().hex[:4]

        # 定义linux终端下是否输出带颜色的文字
        message = super(DefaultServerFormatter, self).format(record)
        from django.conf import settings
        if record.levelname in self.COLORS and settings.ENVIRONMENT in ('dev', 'dist', 'test'):
            message = self.COLORS[record.levelname] + message + NONE
        return message


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/9/10 2:11 下午
# @Author  : Jieay
# @File    : log.py
# Logging setting


class CustomLogConfig(object):

    def get_logging(self, level):
        # settings LOGGING
        logging = {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters': {
                'verbose': {
                    'format': '%(levelname)s [%(asctime)s] %(pathname)s %(lineno)d %(funcName)s %(process)d '
                              '%(thread)d \n \t %(message)s \n',
                    'datefmt': '%Y-%m-%d %H:%M:%S'
                },
                'main': {
                    'datefmt': '%Y-%m-%d %H:%M:%S',
                    'format': '%(asctime)s [%(module)s %(levelname)s] %(message)s',
                },
                'simple': {
                    'format': "[%(levelname)s %(asctime)s %(process)d %(name)s %(module)s:%(lineno)d] %(message)s",
                    'datefmt': '%Y-%m-%d %H:%M:%S'
                },
                'msg': {
                    'format': '%(message)s'
                },
                "custom_log": {
                    "()": "utils.comm.logapi.DefaultServerFormatter",
                    "format": "%(levelname)1.1s [%(process)s-%(request_id)s %(asctime)s %(ip)s %(user)s "
                              "%(name)s|%(lineno)s] %(message)s",
                    "datefmt": "%Y-%m-%d %H:%M:%S"
                }
            },
            'handlers': {
                "console": {
                    "level": level,
                    "class": "logging.StreamHandler",
                    # "formatter": 'simple',
                    # "formatter": 'verbose',
                    "formatter": 'custom_log',
                    "stream": "ext://sys.stdout"
                }
            },
            'loggers': {
                "django.server": {
                    "level": level,
                    "handlers": ["console"],
                    "propagate": False
                },
                "django.request": {
                    "level": level,
                    "handlers": ["console"],
                    "propagate": False
                },
                "app": {
                    "level": level,
                    "handlers": ["console"],
                    "propagate": False
                },
            }
        }
        return logging

    def set_logging(self, level, loggers_app):
        logging = self.get_logging(level)

        loggers_comm = {
            "level": level,
            "handlers": ["console"],
            "propagate": False
        }

        for log_app in loggers_app:
            logging['loggers'][log_app] = loggers_comm

        return logging



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 18:29
# @Author  : Jieay
# @File    : members_view.py
from rest_framework import status
from rest_framework.authentication import get_authorization_header
from rest_framework.authtoken.models import Token
from django_filters import rest_framework as filters
from rest_framework.filters import OrderingFilter
from drf_yasg.utils import swagger_auto_schema
from drf_yasg import openapi
from utils.frame.model_view_set import CustomBaseViewSet, CustomBaseJsonResponse
from utils.frame.api_view_external import BaseAdminApiView, BaseOpenApiView, BaseResponse
from app.models.user.members import Members
from app.serializers.user.members_serializer import MembersModelSerializer, CreateMembersModelSerializer, \
    UpdateMembersModelSerializer, GetMembersSerializer
from utils.comm.commapi import get_serializer_data_to_json, clean_response_field, check_model_field
from utils.frame.pagination import DrfYasgDjangoRestResponsePagination
from utils.user.membersapi import get_aksk_token

import logging
logger = logging.getLogger(__name__)


class MembersView(BaseAdminApiView):
    """成员信息"""

    member_response = openapi.Response('Member list response', GetMembersSerializer)

    def get_token(self, request):
        auth = get_authorization_header(request).split()
        if len(auth) == 2:
            key = auth[1].decode()
        else:
            key = None
        token = Token.objects.select_related('user').get(key=key)
        return token

    def get_user_info(self, token):
        data = {}
        v_fields = [
            'id', 'username', 'name', 'email', 'unit_name', 'dept_name', 'employee_code',
            'employee_name', 'oaid', 'post_name', 'telephone'
        ]
        ck_obj = Members.objects.filter(id=token.user.id)
        if ck_obj.exists():
            data.update(ck_obj.values(*v_fields)[0])
        return data

    @swagger_auto_schema(operation_description="用户信息接口",
                         responses={200: member_response, 404: 'Data not found.'},)
    def get(self, request):
        token = self.get_token(request)
        data = {}
        if token is not None:
            data = self.get_user_info(token)
            return BaseResponse(code=200, data=data, status=True, msg='Queried successfully.')
        return BaseResponse(data=data)


class MembersFilter(filters.FilterSet):
    username = filters.CharFilter(field_name='username', lookup_expr='exact')
    name = filters.CharFilter(field_name='name', lookup_expr='icontains')
    oaid = filters.CharFilter(field_name='oaid', lookup_expr='exact')
    telephone = filters.CharFilter(field_name='telephone', lookup_expr='exact')

    class Meta:
        model = Members
        fields = [
            'id', 'username', 'name', 'oaid', 'telephone'
        ]
        ordering_fields = '__all__'


class MembersViewsSet(CustomBaseViewSet):
    """用户信息"""

    serializer_class = MembersModelSerializer
    queryset = Members.objects.all()
    filter_backends = (filters.DjangoFilterBackend, OrderingFilter)
    filter_class = MembersFilter
    ordering = ['-ctime']
    model_name = Members

    param_id = openapi.Parameter(
        "id", openapi.IN_QUERY, description="筛选参数：ID", type=openapi.TYPE_INTEGER, required=True
    )
    param_username = openapi.Parameter(
        "username", openapi.IN_QUERY, description="筛选参数：用户", type=openapi.TYPE_STRING
    )
    param_name = openapi.Parameter("name", openapi.IN_QUERY, description="筛选参数：姓名", type=openapi.TYPE_STRING)
    param_oaid = openapi.Parameter("oaid", openapi.IN_QUERY, description="筛选参数：OAID", type=openapi.TYPE_STRING)
    param_telephone = openapi.Parameter(
        "telephone", openapi.IN_QUERY, description="筛选参数：手机号码", type=openapi.TYPE_NUMBER
    )

    @swagger_auto_schema(operation_description="用户列表接口",
                         responses={404: 'Data not found.'},
                         manual_parameters=[param_id, param_username, param_name, param_oaid, param_telephone],
                         paginator_inspectors=[DrfYasgDjangoRestResponsePagination]
                         )
    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            data = get_serializer_data_to_json(serializer.data)
            data = clean_response_field(data, self.del_fields)
            return self.get_paginated_response(data)

        serializer = self.get_serializer(queryset, many=True)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data)

    @swagger_auto_schema(operation_description="用户新增接口",
                         responses={400: 'Created failed.'},
                         request_body=CreateMembersModelSerializer
                         )
    def create(self, request, *args, **kwargs):
        self.serializer_class = CreateMembersModelSerializer
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        logger.info('Create: {}'.format(serializer.data))
        data = get_serializer_data_to_json(serializer.data)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_201_CREATED,
                                      headers=headers,
                                      msg="Created successfully.",
                                      status=True)

    @swagger_auto_schema(operation_description="用户修改接口",
                         responses={400: 'Updated failed.'},
                         request_body=UpdateMembersModelSerializer)
    def update(self, request, *args, **kwargs):
        self.serializer_class = UpdateMembersModelSerializer
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, '_prefetched_objects_cache', None):
            instance._prefetched_objects_cache = {}
        logger.info('Update: {}'.format(serializer.data))
        data = get_serializer_data_to_json(serializer.data)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Updated successfully.",
                                      status=True)

    @swagger_auto_schema(operation_description="用户详情接口")
    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        data = get_serializer_data_to_json(serializer.data)
        data = clean_response_field(data, self.del_fields)
        return CustomBaseJsonResponse(data=data,
                                      code=status.HTTP_200_OK,
                                      msg="Queried successfully.",
                                      status=True)

    @swagger_auto_schema(operation_description="用户删除接口",
                         request_body=openapi.Schema(
                             type=openapi.TYPE_OBJECT,
                             required=['username'],
                             properties={
                                 'username': openapi.Schema(type=openapi.TYPE_STRING)
                             }),
                         responses={200: 'Deleted successfully.'}
                         )
    def destroy(self, request, *args, **kwargs):
        check_model_field(request.data, self.model_name, ck_field='username')
        instance = self.get_object()
        _id = instance.id
        self.perform_destroy(instance)
        logger.info('Delete: {}'.format([{"id": _id}]))
        return CustomBaseJsonResponse(data=[{"id": _id}],
                                      code=status.HTTP_200_OK,
                                      msg="Deleted successfully.",
                                      status=True)


class APIGetTokenView(BaseOpenApiView):
    """获取token"""

    @swagger_auto_schema(
        operation_description="获取token接口",
        responses={
            400: 'Queried failed.',
            200: openapi.Schema(
                type=openapi.TYPE_OBJECT,
                properties={
                    'token': openapi.Schema(type=openapi.TYPE_STRING, description="token"),
                }
            )
        },
        request_body=openapi.Schema(
            type=openapi.TYPE_OBJECT,
            required=['accesskey', 'secretkey'],
            properties={
                'accesskey': openapi.Schema(type=openapi.TYPE_STRING, description="ACCESSKEY参数"),
                'secretkey': openapi.Schema(type=openapi.TYPE_STRING, description="SECRETKEY参数"),
            }
        )
    )
    def post(self, request):
        access_key = request.data.get('accesskey')
        secret_key = request.data.get('secretkey')
        token = get_aksk_token(access_key, secret_key)
        data = {}
        if token is not None:
            data['token'] = token
            return BaseResponse(code=200, data=data, status=True, msg='Queried successfully.')
        return BaseResponse(data=data)


# -*- coding: utf-8 -*-
"""
Django settings for frame_server project.

Generated by 'django-admin startproject' using Django 2.2.2.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os
import configparser
config = configparser.ConfigParser()

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
RESOURCE_ROOT = os.path.join(BASE_DIR, 'data')

ENVIRONMENT = os.environ.get('SERVER_CFG', 'dev')

config.read(os.path.join(BASE_DIR, 'config_%s' % ENVIRONMENT))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '$br+r_=i40pe_2x*)2jb(tsy9kt=lagzvse#7oat$osf4@c^&-'

# SECURITY WARNING: don't run with debug turned on in production!
if ENVIRONMENT == 'dev':
    DEBUG = True
else:
    DEBUG = False

ALLOWED_HOSTS = ["*"]

AUTH_USER_MODEL = 'app.Members'
# Application definition

INSTALLED_APPS = [
    'simpleui',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # 第三方依赖库
    'rest_framework',
    'rest_framework.authtoken',
    'django_filters',
    'corsheaders',
    'django_cas_ng',
    'django_celery_beat',
    'django_celery_results',
    'drf_yasg',
    'app',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'utils.comm.middleware.GlobalRequestMiddleware.GlobalRequestMiddleware'
]

ROOT_URLCONF = 'frame_server.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'frame_server.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DB_ENGINE = config['db']['engine']
DB_NAME = config['db']['name']
DB_HOST = config['db']['host']
DB_PORT = config['db']['port']
DB_USER = config['db']['user']
DB_PASSWORD = config['db']['password']

# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.{}'.format(DB_ENGINE),
#         'NAME': DB_NAME,
#         'HOST': DB_HOST,
#         'PORT': DB_PORT,
#         'USER': DB_USER,
#         'PASSWORD': DB_PASSWORD
#     }
# }

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'zh-Hans'

TIME_ZONE = 'Asia/Shanghai'

USE_I18N = True

USE_L10N = True

USE_TZ = False


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = os.path.join(BASE_DIR, "data", "static")
if not os.path.exists(STATIC_ROOT):
    os.makedirs(STATIC_ROOT)


# 配置跨域
CORS_ALLOW_CREDENTIALS = True
CORS_ORIGIN_ALLOW_ALL = True
CORS_ORIGIN_WHITELIST = ()

CORS_ALLOW_METHODS = (
    'DELETE',
    'GET',
    'OPTIONS',
    'PATCH',
    'POST',
    'PUT',
    'VIEW',
)

CORS_ALLOW_HEADERS = (
    'XMLHttpRequest',
    'X_FILENAME',
    'accept',
    'accept-encoding',
    'authorization',
    'content-type',
    'dnt',
    'origin',
    'user-agent',
    'x-csrftoken',
    'x-requested-with',
    'Pragma',
)

# Logging setting
LOG_LEVEL = config['base']['log_level']
if LOG_LEVEL not in ['DEBUG', 'INFO']:
    LOG_LEVEL = 'DEBUG'

loggers_app = [
    "utils.frame",
]
from .log import CustomLogConfig
LOGGING = CustomLogConfig().set_logging(LOG_LEVEL, loggers_app)


# 配置异步
from kombu import Queue
REDIS_HOST = config['redis']['rd_host']
REDIS_PASSWORD = config['redis']['rd_password']
REDIS_PORT = config['redis']['rd_port']
REDIS_BASE_DB = config['redis']['base_db']

# Redis 连接池 DB
REDIS_CONSTANCE_DB = config['redis']['constance_db']

# backer 存储生产者消息 DB
REDIS_BROKER_DB = config['redis']['broker_db']

# celery 缓存 DB
REDIS_CELERY_CACHE_DB = config['redis']['celery_cache_db']
# celery 结果存储 DB
REDIS_CELERY_RESULT_DB = config['redis']['celery_result_db']

# redis 连接池
CONSTANCE_REDIS_CONNECTION = f"redis://:{REDIS_PASSWORD}@{REDIS_HOST}:{REDIS_PORT}/{REDIS_CONSTANCE_DB}"

# backer 存储生产者消息
BROKER_URL = f"redis://:{REDIS_PASSWORD}@{REDIS_HOST}:{REDIS_PORT}/{REDIS_BROKER_DB}"

# celery 定时任务中间件
CELERYBEAT_SCHEDULER = "django_celery_beat.schedulers.DatabaseScheduler"

# celery 处理结果保存
CELERY_RESULT_BACKEND = f'redis://:{REDIS_PASSWORD}@{REDIS_HOST}:{REDIS_PORT}/{REDIS_CELERY_RESULT_DB}'

# celery 缓存
CELERY_CACHE_BACKEND = f'redis://:{REDIS_PASSWORD}@{REDIS_HOST}:{REDIS_PORT}/{REDIS_CELERY_CACHE_DB}'


# 队列配置
CELERY_QUEUES = (
    # celery 默认队列
    Queue('celery', routing_key='celery'),
    # demo
    Queue('notify', routing_key='notify'),
)
# 路由
CELERY_ROUTES = {
    '*.notify_queue*': {'queue': 'notify'},
}

CELERY_ACCEPT_CONTENT = ['application/json']
CELERY_TASK_SERIALIZER = 'json'
CELERY_RESULT_SERIALIZER = 'json'
# celery时区设置，使用settings中TIME_ZONE同样的时区
CELERY_TIMEZONE = TIME_ZONE
# celery 禁用 UTC 和 TZ
CELERY_ENABLE_UTC = False
DJANGO_CELERY_BEAT_TZ_AWARE = False


# 后端认证
AUTHENTICATION_BACKENDS = [
    'django_cas_ng.backends.CASBackend',
    'django.contrib.auth.backends.ModelBackend',
]

# rest framework 配置
REST_FRAMEWORK = {
    # 配置分页，使用按偏移量分页
    'DEFAULT_PAGINATION_CLASS': 'utils.frame.pagination.CustomBasePagination',
    'PAGE_SIZE': 10,
    'EXCEPTION_HANDLER': 'utils.frame.exception_handler.CustomBaseExceptionHandler',

    # API接口权限验证，配置只有注册用户才可通过验证, AllowAny不做权限限制.
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),

    # 配置单点系统
    'DEFAULT_MIDDLEWARE_CLASSES': (
        'django_cas_ng.middleware.CASMiddleware',
    ),

    # 配置认证
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.TokenAuthentication',
        'rest_framework.authentication.BasicAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        # 'django_cas_ng.backends.CASBackend',
        'django.contrib.auth.backends.ModelBackend',
    ),
}


# CAS 的地址
# CAS_SERVER_URL = 'http://cas-g2.xxxxx.com.cn'    # 新域名
CAS_SERVER_URL = config['cas']['cas_server_url']   # 老域名

# CAS 版本
CAS_VERSION = '3'


# 存入所有 CAS 服务端返回的 User 数据。
CAS_APPLY_ATTRIBUTES_TO_USER = True
# 自定义返回值匹配用户表
# {'ln':'last_name'}，cas服务器返回的ln属性将重命名为last_name
CAS_RENAME_ATTRIBUTES = {
    'name': 'name',
    'unitname': 'unit_name',
    'deptname': 'dept_name',
    'employeeCode': 'employee_code',
    'employeeName': 'employee_name',
    'oaid': 'oaid',
    'postname': 'post_name',
    'telephone': 'telephone'
}

# CAS 自定义额外参数，用于在请求服务端返回指定json格式
CAS_EXTRA_LOGIN_PARAMS = {'format': 'json'}
CAS_RENEW = True

# CAS 自动创建用户
CAS_AUTO_CREATE_USERS = True


# Django 默认的数据存储根目录， model的 upload_to 字段用到
MEDIA_ROOT = RESOURCE_ROOT
MEDIA_URL = '/media/'


# 企业微信信息
CORP_ID = config['weixin']['corp_id']
CORP_SECRET = config['weixin']['corp_secret']
AGENT_ID = config['weixin']['agent_id']


SWAGGER_SETTINGS = {
    # 基础样式
    'SECURITY_DEFINITIONS': {
        "basic": {
            'type': 'basic'
        }
    },
    # 如果需要登录才能够查看接口文档, 登录的链接使用restframework自带的.
    'LOGIN_URL': '/admin/login',
    'LOGOUT_URL': '/admin/logout',
    # 'DOC_EXPANSION': None,
    # 'SHOW_REQUEST_HEADERS':True,
    # 'USE_SESSION_AUTH': True,
    # 'DOC_EXPANSION': 'list',
    # 接口文档中方法列表以首字母升序排列
    'APIS_SORTER': 'alpha',
    # 如果支持json提交, 则接口文档中包含json输入框
    'JSON_EDITOR': True,
    # 方法列表字母排序
    'OPERATIONS_SORTER': 'alpha',
    'VALIDATOR_URL': None,
    # 'DEFAULT_PAGINATOR_INSPECTORS': [
    #     'utils.frame.pagination.DrfYasgDjangoRestResponsePagination',
    #     'drf_yasg.inspectors.CoreAPICompatInspector',
    # ]
}


# web URL
WEB_BASE_URL = config['base']['web_base_url']




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 18:21
# @Author  : Jieay
# @File    : urls.py

from django.conf.urls import url, include, re_path
from rest_framework import routers


urlpatterns = []

router = routers.DefaultRouter()

from app.views.user.members_view import MembersViewsSet, MembersView
from app.views.comm.comm_view import GetWebUrlView

router.register(r'users', MembersViewsSet)


urlpatterns += [
    re_path(r'v1/members/list', MembersView.as_view()),
    re_path(r'v1/web/url', GetWebUrlView.as_view()),
]


urlpatterns += [
    url(r'v1/', include(router.urls)),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/9/10 2:36 下午
# @Author  : Jieay
# @File    : GlobalRequestMiddleware.py

import threading
import uuid
from django.utils.deprecation import MiddlewareMixin


threading_local = threading.local()


class GlobalRequestMiddleware(MiddlewareMixin):
    """
    全局request中间件
    """

    def process_request(self, request):
        request.id = self.generate_request_id()
        request.ip = request.META.get('HTTP_X_FORWARDED_FOR') or request.META.get('REMOTE_ADDR')
        threading_local.request = request

    @staticmethod
    def generate_request_id():
        return uuid.uuid4().hex[:4]


class GlobalRequest(object):

    def __getattr__(self, item):
        return getattr(getattr(threading_local, 'request', None), item, None)


global_request = GlobalRequest()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/6/15 10:13 上午
# @Author  : Jieay
# @File    : sub_urls
# 子服务路由 url 注册
from django.conf.urls import url, include, re_path


urlpatterns = []


# -*- coding: utf-8 -*-

import ast
import json
from django.db import models


class DictField(models.TextField):
    description = "Stores a python dict"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def to_python(self, value):
        if not value:
            value = {}

        if isinstance(value, dict):
            return value

        return ast.literal_eval(value)

    def get_prep_value(self, value):
        if value is None:
            return value

        return str(value)  # use unicode(value) in Python 2

    def _get_val_from_obj(self, obj):
        if obj is not None:
            return getattr(obj, self.attname)
        else:
            return self.get_default()

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-09-23 17:23
# @Author  : Jieay
# @File    : demo_test.py
from frame_server.celery import celery_app as app
from celery import shared_task
import time


# 异步任务
# 使用方式：add.delay(2,8)
@app.task
def add(x, y):
    time.sleep(15)
    return x + y


# 定时任务两种写法
# 未指定队列，使用默认队列，在分布式中所有worker都可执行
@shared_task()
def timed_test():
    msg = f'测试定时任务'
    return msg


# 指定队列，在分布式中只在demo队列的节点上执行，如未启动demo队列worker，则不执行
@app.task(queue='demo', name='app.tasks.demo_test.demo_queue_test')
def demo_queue_test():
    msg = f'指定队列，测试定时任务'
    return msg




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:31
# @Author  : Jieay
# @File    : fake_model.py
from django.db import models


class FakeModel(models.Model):
    pass

    class Meta:
        db_table = "app_common_fake"



"""frame_server URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import path, re_path, include
from django.conf import settings

from rest_framework import permissions
from drf_yasg.views import get_schema_view
from drf_yasg import openapi


DRF_TITLE = "PDF脚手架"

schema_view = get_schema_view(
   openapi.Info(
      title="{} API".format(DRF_TITLE),
      default_version='v1',
      description="项目接口文档",
      terms_of_service="https:xxx.xxx.com.cn",
      contact=openapi.Contact(email="1016900854@qq.com"),
      license=openapi.License(name="BSD License"),
   ),
   public=True,
   permission_classes=(permissions.AllowAny,),
)


from app.views.cas_admin import views as cas_views
from app.views.comm.health_view import GetHealthCheckView
from app.views.user.members_view import APIGetTokenView
urlpatterns = [
    path('admin/', admin.site.urls),
    path('gettoken', APIGetTokenView.as_view()),
    path('login/', cas_views.CasLoginView.as_view(), name='cas_ng_login'),
    path('logout/', cas_views.CasLogoutView.as_view(), name='cas_ng_logout'),
    re_path(r'health/check', GetHealthCheckView.as_view()),
]

if settings.DEBUG is True:
    urlpatterns += [
        re_path(r'^swagger(?P<format>\.json|\.yaml)$', schema_view().without_ui(cache_timeout=0), name='schema-json'),
        re_path(r'^swagger/$', schema_view().with_ui('swagger', cache_timeout=0), name='schema-swagger-ui'),
        re_path(r'^redoc/$', schema_view().with_ui('redoc', cache_timeout=0), name='schema-redoc'),
    ]

urlpatterns += [
    re_path(r'^api/', include('app.urls')),
]


# -*- coding: utf-8 -*-

from django.db import models
import ast


class ListField(models.TextField):
    description = "Stores a python list"

    def __init__(self, *args, **kwargs):
        super(ListField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if not value:
            value = []

        if isinstance(value, list):
            return value

        return ast.literal_eval(value)

    def get_prep_value(self, value):
        if value is None:
            return value

        return str(value)  # use unicode(value) in Python 2

    # @warn_about_renamed_method(
    #     'Field', '_get_val_from_obj', 'value_from_object',
    #     RemovedInDjango20Warning
    # )
    def _get_val_from_obj(self, obj):
        if obj is not None:
            return getattr(obj, self.attname)
        else:
            return self.get_default()

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-06 12:51
# @Author  : Jieay
# @File    : comm_view.py
from drf_yasg.utils import swagger_auto_schema
from drf_yasg import openapi
from utils.frame.api_view_external import BaseAdminApiView, BaseResponse
from django.conf import settings
from utils.frame.pagination import DrfYasgDjangoRestResponsePagination

import logging
logger = logging.getLogger(__name__)


class GetWebUrlView(BaseAdminApiView):

    data_response = openapi.Response('Web URL response', openapi.Schema('web_url', type=openapi.TYPE_STRING))

    @swagger_auto_schema(operation_description="前端地址查询",
                         responses={200: data_response, 404: 'Data not found.'},
                         paginator_inspectors=[DrfYasgDjangoRestResponsePagination])
    def get(self, request):
        web_url = settings.WEB_BASE_URL
        data = {}
        if web_url:
            data['web_url'] = web_url
            return BaseResponse(code=200, data=data, status=True, msg='Queried successfully.')
        return BaseResponse(data=data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 17:32
# @Author  : Jieay
# @File    : fake_serializer.py

from rest_framework import serializers
from app.models.common.fake_model import FakeModel


class FakeSerializer(serializers.ModelSerializer):
    class Meta:
        model = FakeModel
        fields = '__all__'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/9/17 15:22
# @Author  : Jieay
# @File    : setup.py

with open("DOCS.md", "r") as fh:
    long_description = fh.read()

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import setuptools


setup(
    name='etu-django-frame',  # 包的名字
    version='1.1.0',  # 版本号
    author="Jieay",
    author_email="1016900854@qq.com",
    description='将PDF脚手架工具，制作成为pip安装包，以方便使用',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='Apache License',
    packages=setuptools.find_packages(),  # 包需要引用的文件夹
    # packages = setuptools.find_packages(exclude=['文件夹名字']),
    include_package_data=True,

    # 依赖包, 应用到的第三方库
    install_requires=[],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries'
    ],
    zip_safe=True,
    entry_points={
        'console_scripts': [
            'edf-admin = etu_django_frame.main:execute_from_command_line',
        ],
    },
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-10-10 17:32
# @Author  : Jieay
# @File    : views.py
from __future__ import absolute_import, unicode_literals
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login as auth_login, logout as auth_logout
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from urllib import parse as urllib_parse
from django.utils.translation import ugettext_lazy as _
from django_cas_ng.models import ProxyGrantingTicket, SessionTicket
from django_cas_ng.signals import cas_user_logout
from django_cas_ng.utils import (
    get_cas_client,
    get_protocol,
    get_redirect_url,
    get_service_url,
)

from django_cas_ng.views import LoginView, LogoutView, clean_sessions
from rest_framework.authtoken.models import Token

import logging
logger = logging.getLogger(__name__)


class CasLoginView(LoginView):

    """单点登录"""

    def successful_login(self, request, next_page):
        """
        This method is called on successful login. Override this method for
        custom post-auth actions (i.e, to add a cookie with a token).

        :param request: WSGIRequest
        :param next_page:
        :return:
        """
        user = request.user
        logger.info('check user: {}'.format(user))
        token_obj = Token.objects.filter(user=user)
        if token_obj.exists():
            token = token_obj[0].key
        else:
            token = None
        logger.info('Token: {}'.format(token))
        session_key = request.session.session_key
        # logger.info('Login request.COOKIES: {}'.format(request.COOKIES))
        # logger.info('Login request.session: {}'.format(request.session))
        # get_coolies = request.COOKIES
        # csrftoken = get_coolies.get('csrftoken')
        if token is not None:
            new_next_page = '{}?token={}&session_key={}'.format(next_page, token, session_key)
            # if settings.DEBUG:
            #     new_next_page = '{}?csrftoken={}&sessionid={}&token={}'.format(
            #         next_page, csrftoken, sessionid, token
            #     )
        else:
            new_next_page = next_page
        logger.info('next_page: {}'.format(new_next_page))

        return HttpResponseRedirect(new_next_page)

    def post(self, request):
        if request.POST.get('logoutRequest'):
            next_page = request.POST.get('next', settings.CAS_REDIRECT_URL)
            service_url = get_service_url(request, next_page)
            client = get_cas_client(service_url=service_url, request=request)

            clean_sessions(client, request)
            return HttpResponseRedirect(next_page)

    def get(self, request):
        """
        Forwards to CAS login URL or verifies CAS ticket

        :param request:
        :return:
        """
        next_page = request.GET.get('next')
        required = request.GET.get('required', False)

        service_url = get_service_url(request, next_page)
        # logger.info('service_url: {}'.format(service_url))
        client = get_cas_client(service_url=service_url, request=request)

        if not next_page and settings.CAS_STORE_NEXT and 'CASNEXT' in request.session:
            next_page = request.session['CASNEXT']
            del request.session['CASNEXT']

        if not next_page:
            next_page = get_redirect_url(request)

        if request.user.is_authenticated:
            if settings.CAS_LOGGED_MSG is not None:
                message = settings.CAS_LOGGED_MSG % request.user.get_username()
                messages.success(request, message)
                logger.info('user is authenticated')
                user = request.user
                Token.objects.update_or_create(user=user)
            return self.successful_login(request=request, next_page=next_page)

        ticket = request.GET.get('ticket')
        logger.info('Login ticket: {}'.format(ticket))
        if ticket:
            user = authenticate(ticket=ticket,
                                service=service_url,
                                request=request)
            logger.info('ticket user: {}'.format(user))
            # print('user:', type(user))
            pgtiou = request.session.get("pgtiou")
            if user is not None:
                if not request.session.exists(request.session.session_key):
                    request.session.create()
                auth_login(request, user)
                SessionTicket.objects.create(
                    session_key=request.session.session_key,
                    ticket=ticket
                )
                Token.objects.update_or_create(user=user)

                if pgtiou and settings.CAS_PROXY_CALLBACK:
                    # Delete old PGT
                    ProxyGrantingTicket.objects.filter(
                        user=user,
                        session_key=request.session.session_key
                    ).delete()
                    # Set new PGT ticket
                    try:
                        pgt = ProxyGrantingTicket.objects.get(pgtiou=pgtiou)
                        pgt.user = user
                        pgt.session_key = request.session.session_key
                        pgt.save()
                    except ProxyGrantingTicket.DoesNotExist:
                        pass

                if settings.CAS_LOGIN_MSG is not None:
                    name = user.get_username()
                    message = settings.CAS_LOGIN_MSG % name
                    messages.success(request, message)
                return self.successful_login(request=request, next_page=next_page)
            elif settings.CAS_RETRY_LOGIN or required:
                return HttpResponseRedirect(client.get_login_url())
            else:
                raise PermissionDenied(_('Login failed.'))
        else:
            if settings.CAS_STORE_NEXT:
                request.session['CASNEXT'] = next_page
            return HttpResponseRedirect(client.get_login_url())


class CasLogoutView(LogoutView):

    """单点退出"""

    def get(self, request):
        """
        Redirects to CAS logout page

        :param request:
        :return:
        """
        next_page = request.GET.get('next')
        token = request.GET.get('token')
        session_key = request.GET.get('session_key')

        logger.info('token: {} session_key: {}'.format(token, session_key))
        # try to find the ticket matching current session for logout signal
        try:
            st = SessionTicket.objects.get(session_key=session_key)
            ticket = st.ticket
        except SessionTicket.DoesNotExist:
            ticket = None
        # send logout signal
        # print('request.COOKIES: {}'.format(request.COOKIES))
        # print('request.session: {}'.format(request.session))
        logger.info('request.user: {}'.format(request.user))
        logger.info('logout ticket: {}'.format(ticket))
        logger.info('Start cas logout.')
        cas_user_logout.send(
            sender="manual",
            user=request.user,
            session=request.session,
            ticket=ticket,
        )
        logger.info('Start sys logout.')
        auth_logout(request)
        # clean current session ProxyGrantingTicket and SessionTicket
        ProxyGrantingTicket.objects.filter(session_key=session_key).delete()
        SessionTicket.objects.filter(session_key=session_key).delete()
        Token.objects.filter(key=token).delete()

        next_page = next_page or get_redirect_url(request)
        logger.info('Logout next_page: {}'.format(next_page))
        if settings.CAS_LOGOUT_COMPLETELY:
            protocol = get_protocol(request)
            host = request.get_host()
            redirect_url = urllib_parse.urlunparse(
                (protocol, host, next_page, '', '', ''),
            )
            logger.info('Logout redirect_url: {}'.format(redirect_url))
            client = get_cas_client(request=request)
            # logger.info('Logout client.get_logout_url(redirect_url): {}'.format(client.get_logout_url(redirect_url)))
            return HttpResponseRedirect(client.get_logout_url(next_page))
        else:
            # This is in most cases pointless if not CAS_RENEW is set. The user will
            # simply be logged in again on next request requiring authorization.
            return HttpResponseRedirect(next_page)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 16:43
# @Author  : Jieay
# @File    : __init__.py.py
from app.models.common.fake_model import *
from app.models.user.members import *


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 18:28
# @Author  : Jieay
# @File    : members_serializer.py

from rest_framework import serializers
from app.models.user.members import Members


class MembersModelSerializer(serializers.ModelSerializer):
    department_info = serializers.ReadOnlyField(help_text="部门列表详细信息字段")

    class Meta:
        model = Members
        fields = '__all__'


class CreateMembersModelSerializer(MembersModelSerializer):

    def __init__(self, *args, **kwargs):
        super(CreateMembersModelSerializer, self).__init__(*args, **kwargs)
        self.fields['oaid'].required = True


class UpdateMembersModelSerializer(CreateMembersModelSerializer):
    pass


class GetMembersSerializer(serializers.ModelSerializer):

    class Meta:
        model = Members
        fields = [
            'id', 'username', 'name', 'email', 'unit_name', 'dept_name', 'employee_code',
            'employee_name', 'oaid', 'post_name', 'telephone'
        ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2019-09-17 08:58
# @Author  : Jieay
# @File    : commapi.py

import os
import re
import sys
import json
import copy
import time
import string
import random
import uuid
import fcntl
import inspect
import datetime
import functools
import subprocess


from rest_framework.exceptions import APIException


def check_legal_python():
    """检查合法python版本, 3.6 <= python < 3.9"""
    version_major = sys.version_info.major
    version_minor = sys.version_info.minor
    if version_major > 2 and 6 <= version_minor < 9:
        return True
    return False


def func_execution_time(func):
    """
    方法执行时间 装饰器
    :param func:
    :return:
    """
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        start = time.perf_counter()
        res = func(*args, **kwargs)
        end = time.perf_counter()
        print('{} took execution time {} ms'.format(func.__name__, (end - start) * 1000))
        return res
    return wrapper


def get_random(mun, ty=None):
    """
    获取随机唯一字符串
    :param mun: 随机字符长度
    :param ty: 头部前缀字符
    :return:
    """
    if ty == 'tyint':
        restr = string.digits
    elif ty == 'tystr':
        restr = string.ascii_letters
    else:
        restr = string.digits + string.ascii_letters

    str_list = []
    for i in range(int(mun)):
        cs = random.choice(restr)
        str_list.append(cs)
    return ''.join(str_list)


def get_unique_id(title=None):
    """
    :param title: unique_id 的头部， 默认无
    :return: str 默认为28位数字
    """
    today_time = datetime.datetime.now()
    date_time = today_time.strftime("%Y%m%d%H%M%S")
    time_stamp = today_time.strftime("%s")
    time_str = str(date_time) + str(time_stamp)
    random_mun = get_random(4, ty='tyint')
    if title:
        str_id = '{0}{1}{2}'.format(str(title), time_str, random_mun)
    else:
        str_id = '{0}{1}'.format(time_str, random_mun)
    return str_id


def str_to_float(data):
    """
    将字符串转化成浮点数
    """
    try:
        data = float(data)
    except:
        data = data
    return data


def datetime_to_str(o):
    """
    封装将 datetime 时间对象转换字符串格式
    :param o: 时间对象：datetime.datetime or datetime.date
    :return:
    """
    if isinstance(o, datetime.datetime):
        return o.strftime('%Y-%m-%d %H:%M:%S')
    elif isinstance(o, datetime.date):
        return o.strftime('%Y-%m-%d')
    else:
        return str(o)


def get_time_stamp(datetime_obj):
    """
    获取时间戳
    :param datetime_obj: 时间对象
    :return:
    """
    if isinstance(datetime_obj, datetime.datetime):
        return int(time.mktime(datetime_obj.timetuple()))
    elif re.match(r'\d+-\d+-\d+ \d+:\d+:\d+', datetime_obj):
        time_strp = time.strptime(datetime_obj, '%Y-%m-%d %H:%M:%S')
        return int(time.mktime(time_strp))
    return


def check_json_format(raw_msg):
    """
    用于判断一个字符串是否符合Json格式
    :param raw_msg: 源字符
    :return: 布尔值
    """
    if raw_msg == 'null':
        return False

    if isinstance(raw_msg, str):
        try:
            if check_legal_python():
                json.loads(raw_msg, encoding='utf-8')
            else:
                json.loads(raw_msg)
        except ValueError:
            return False
        return True
    else:
        return False


def get_serializer_data_to_json(serializer_data):
    data = copy.deepcopy(serializer_data)
    if isinstance(data, dict):
        for k, v in data.items():
            if check_json_format(v):
                serializer_data[k] = json.loads(v)
            if isinstance(v, dict):
                for kk, vv in v.items():
                    if check_json_format(vv):
                        serializer_data[k][kk] = json.loads(vv)

    if isinstance(data, list):
        for nu, me in enumerate(data):
            if isinstance(me, dict):
                for k, v in me.items():
                    if check_json_format(v):
                        serializer_data[nu][k] = json.loads(v)
                    if isinstance(v, dict):
                        for kk, vv in v.items():
                            if check_json_format(vv):
                                serializer_data[nu][k][kk] = json.loads(vv)
    return serializer_data


class ApiLimitException(APIException):
    status_code = 503
    default_detail = 'Task already exists, try again later.'


class ApiFieldCheckException(APIException):
    status_code = 507
    default_detail = 'Field check exception.'


def api_limit_from_db(model, ck_field, key=None):
    """
    通过查询数据库指定字段条件，限制API接口请求
    :param model: 数据库model
    :param ck_field: 获取 request.data 查询的字段名称
    :param key: 自定义字段筛选条件，类型为 dict
    """
    def decorate(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            assert isinstance(key, dict), ApiLimitException(
                detail="@api_limit_from_db() missing required argument: 'key'"
            )

            request_data = args[1].data
            db_key = {}
            if isinstance(ck_field, (list, tuple)):
                ck_field_list = ck_field
            else:
                ck_field_list = [ck_field]
            for i in ck_field_list:
                db_field = request_data.get(i)
                if db_field is not None:
                    db_key[i] = db_field
            assert db_key, ApiLimitException(
                detail="api_limit_from_db() missing required data: 'ck_field'"
            )

            if isinstance(key, dict):
                key.update(db_key)
                obj = model.objects.filter(**key)
                for ok, ov in db_key.items():
                    key.pop(ok)
                if obj:
                    the_object = True
                else:
                    the_object = False
            else:
                the_object = False
            assert not the_object, ApiLimitException(
                detail="@api_limit_from_db() check db task already exists."
            )

            res = func(*args, **kwargs)
            return res
        return wrapper
    return decorate


def check_request_field(data, ck_field):
    """
    检查请求中是否存在指定的字段
    :param data: 请求数据
    :param ck_field: 指定的字段，可以是多个
    """
    if not isinstance(data, dict):
        raise ApiFieldCheckException(detail='The request is not JSON.')
    field_list = []
    if isinstance(ck_field, (list, tuple)):
        field_list.extend([x for x in ck_field])
    else:
        field_list.append(ck_field)

    ck_field_list = copy.deepcopy(field_list)
    for i in ck_field_list:
        ck_data_field = data.get(i, None)
        if ck_data_field is not None:
            field_list.remove(i)
    if field_list:
        msg_filed = ','.join(field_list)
        raise ApiFieldCheckException(detail='The request data {} is not exist.'.format(msg_filed))

    return data


def check_model_field(data, model, **kwargs):
    """
    检查请求的数据，指定字段是否存在数据库中，is_exist: 默认不存在则报异常，否则存在报异常
    :param data: 请求数据
    :param model: 数据库 model
    :param kwargs: ck_field 指定检查的字段，可以是多个
    """
    field_list = []
    exclude_fields = ['ck_field', 'is_exist', 'check_del']
    ck_field = kwargs.get('ck_field', None)
    if isinstance(ck_field, (list, tuple)):
        field_list.extend(ck_field)
    if ck_field:
        field_list.append(ck_field)
    if isinstance(kwargs, dict):
        _ks = list(kwargs.keys())
        for ef in exclude_fields:
            if ef in _ks:
                _ks.remove(ef)
        field_list.extend(_ks)
    check_request_field(data=data, ck_field=field_list)

    ck_dict = {}
    for i in field_list:
        fd = data.get(i, None)
        if fd:
            ck_dict[i] = fd

    is_exist = kwargs.get('is_exist', False)
    check_del = kwargs.get('check_del', None)
    if check_del:
        ck_dict['soft_del'] = False

    ck_obj = model.objects.filter(**ck_dict)
    if is_exist is True:
        if ck_obj.exists():
            msg_filed = ', '.join(ck_dict)
            raise ApiFieldCheckException(detail='The request data {} is exist for model.'.format(msg_filed))
    else:
        if not ck_obj.exists():
            msg_filed = ', '.join(ck_dict)
            raise ApiFieldCheckException(detail='The request data {} is not exist for model.'.format(msg_filed))

    return data


def clean_response_field(data, ck_field):
    """
    删除请求返回值中的字段
    :param data: 返回数据
    :param ck_field: 需要删除的字段
    """
    data_cp = copy.deepcopy(data)

    if not isinstance(data, (dict, list)):
        raise ApiFieldCheckException(detail='The response is not JSON.')
    field_list = []
    if isinstance(ck_field, (list, tuple)):
        field_list.extend([x for x in ck_field])
    else:
        field_list.append(ck_field)

    if isinstance(data, dict):
        for k, v in data_cp.items():
            if k in field_list:
                data.pop(k)
    if isinstance(data, list):
        for m, obj in enumerate(data_cp):
            if isinstance(obj, dict):
                for kk, vv in obj.items():
                    if kk in field_list:
                        data[m].pop(kk)
    return data


def get_model_field_data(model, ck_field, *args, **kwargs):
    """
    获取指定字段的数据，并且可以指定字段排序
    :param model: 数据库 model
    :param ck_field: 自定字段，单个或者一个列表
    :param args: 指定字段，多个字段
    :param kwargs: 排序字段order_by， 正序'xxx'|倒序'-xxx'
    """
    filed_list = []
    if isinstance(ck_field, (list, tuple)):
        filed_list.extend(ck_field)
    else:
        filed_list.append(ck_field)
    for i in args:
        if isinstance(i, (list, tuple)):
            filed_list.extend(i)
        else:
            filed_list.append(i)
    order_by = kwargs.get('order_by')
    if order_by is not None:
        ck_obj = model.objects.all().order_by(order_by).values(*filed_list)
    else:
        ck_obj = model.objects.all().values(*filed_list)
    return list(ck_obj)


def check_dict_fields(data, ck_fields):
    """
    检查数据字典中，查询的字段不存在值则 返回 False
    :param data: 数据字典
    :param ck_fields: 查询字段 列表
    :return: False or True
    """
    if not isinstance(ck_fields, (list, tuple)):
        ck_fields = [ck_fields]

    _info = copy.deepcopy(ck_fields)
    if isinstance(data, dict):
        for i in ck_fields:
            ck_v = data.get(i, None)
            if ck_v is not None:
                _info.remove(i)
    if _info:
        return False
    return True


def str_to_int(data):
    """
    将字符串转化成数字
    :param data: str
    :return: int
    """
    try:
        data_info = int(data)
    except:
        data_info = data
    return data_info


def get_str_to_class(data):
    """
    解析对象字符串，将字符串转化成类名称和包的路径
    :param data: 对象字符串
    :return: (类名称, 包路径)
    """
    # data = "<class 'app.models.asset.envs_model.EnvsModel'>"
    data = data.replace('<', '').replace('>', '').replace("'", "").split(' ')
    if len(data) == 2:
        path_str = None
        m_name = None
        if data[0] == 'class':
            p_k = data[1].split('.')
            path_str = '.'.join(p_k[:-1])
            m_name = p_k[-1]
        return m_name, path_str
    return


def get_model_data_from_id(_id, model):
    """
    通过 ID 获取数据库中该条记录的数据
    :param _id: ID
    :param model: 表对象
    :return: `dict`
    """
    obj = model.objects.filter(id=_id).values()
    return obj[0]


def set_mark_delete(_id, model, **kwargs):
    """
    数据软删除，更新数据的标记记录
    :param _id: ID
    :param model: model
    :param kwargs: soft_del=True
    """
    if not kwargs:
        kwargs['soft_del'] = True
    obj = model.objects.filter(id=_id).update(**kwargs)
    return obj


def set_base_response_data(data=None):
    """
    处理自定义接口返回值的 data 数据中的状态字段
    """
    _data = {}
    if isinstance(data, dict):
        _data = copy.deepcopy(data)
        for i in ['status', 'code', 'msg']:
            if data.get(i, None) is not None:
                _data.pop(i)
    else:
        if data is not None:
            _data = data
    return _data


def check_telephone(telephone):
    """
    检查手机号码的合法性
    :param telephone: 手机号字符
    """
    regx = re.match('^[1-9][1-9][0-9]{9}$', telephone)
    if regx is None:
        return False
    return True


def modify_dict_mapping_dict(data, mapping_dict):
    """
    实现将一个字典的key映射成另外的key
    :param data: 要进行处理的数据字典，旧数据字典
    :param mapping_dict: 映射表，字典类型
    :return: 进过修改key名字的新数据字典
    """
    new_data = {}
    # 判断是否是JSON格式
    if check_json_format(data):
        data = json.loads(data)

    # 判断传入的参数是否为dict类型
    if not isinstance(data, dict):
        return new_data
    if not isinstance(mapping_dict, dict):
        return new_data
    for d_k, d_v in data.items():
        if check_json_format(d_v):
            d_v = json.loads(d_v)
        n_d_v = copy.deepcopy(d_v)
        # if isinstance(d_v, list):
        #     if d_v and isinstance(d_v[0], dict):
        #         n_d_v = d_v[0].get('name')

        if isinstance(n_d_v, (list, dict, tuple)):
            n_d_v = json.dumps(n_d_v)

        new_ks = mapping_dict.get(d_k)
        if isinstance(new_ks, list):
            for kk in new_ks:
                new_data[kk] = n_d_v
        else:
            if new_ks:
                new_data[new_ks] = n_d_v
    return new_data


def remove_fields_not_exists_model(model, dict_data):
    """
    删除不存在model字段中的dict keys
    :param model: 数据库模型
    :param dict_data: 字典对象
    :return:
    """
    field_names = [f.name for f in model._meta.fields]
    temp_dict = copy.copy(dict_data)
    for key in temp_dict.keys():
        if key not in field_names:
            dict_data.pop(key)


def order_time_mg(location_datetime, time_type=None, leadtime=10, timeout=10):
    """
    订单超时或者提前时间管理
    :param location_datetime: 指定时间
    :param time_type: 时间类型，超时或者提前，[ leadtime | timeout ]
    :param leadtime: 提前几分钟， 默认提前10分钟
    :param timeout: 超时几分钟，默认10分钟
    :return: bool
    """
    ck_time = False
    location_stamp = get_time_stamp(location_datetime)

    if time_type == 'timeout':
        # 超时
        timeout_limit_datetime = datetime.datetime.now() - datetime.timedelta(minutes=timeout)
        timeout_limit_stamp = get_time_stamp(timeout_limit_datetime)
        if timeout_limit_stamp > location_stamp:
            ck_time = True
    elif time_type == 'leadtime':
        # 提前
        leadtime_limit_datetime = datetime.datetime.now() + datetime.timedelta(minutes=leadtime)
        leadtime_limit_stamp = get_time_stamp(leadtime_limit_datetime)
        if leadtime_limit_stamp > location_stamp:
            ck_time = True

    return ck_time


def get_function_name(c_name=None):
    """
    动态获取正在运行函数(或方法)名称
    :param c_name: 类名称
    """
    if c_name is not None:
        f_name = '%s.%s' % (c_name, inspect.stack()[1][3])
    else:
        f_name = '%s' % inspect.stack()[1][3]
    return f_name


def func_exe_retry(ExceptionToCheck, tries=4, delay=3, back_off=2, logger=None):
    """
    函数运行重试装饰器
    :param ExceptionToCheck: 异常参数元组（TypeError,ValueError）
    :param tries: 重试次数
    :param delay: 重试延时时间 （秒）
    :param back_off: 重试延时倍数
    :param logger: 日志参数
    """
    def deco_retry(f):
        @functools.wraps(f)
        def f_retry(*args, **kwargs):
            m_tries, m_delay = tries, delay
            while m_tries > 1:
                try:
                    return f(*args, **kwargs)
                except ExceptionToCheck as e:
                    msg = "%s, Retrying in %d seconds..." % (str(e), m_delay)
                    if logger:
                        logger.warning(msg)
                    else:
                        print(msg)
                    time.sleep(m_delay)
                    m_tries -= 1
                    m_delay *= back_off
            return f(*args, **kwargs)
        return f_retry  # true decorator
    return deco_retry


def random_sleep_ms():
    """随机毫秒休眠"""
    sleep_time = random.randint(0, 100) * 0.01
    time.sleep(sleep_time)
    return sleep_time


def filter_attrs(attrs):
    for k, v in attrs.items():
        if k.startswith("__"):
            yield k, v
        else:
            if isinstance(v, tuple):
                yield k, v[0]
                yield f'{k}_NAME', v[1]
            else:
                yield k, v


class ChoicesMeta(type):
    def __new__(cls, name, bases, attrs):
        real_attrs = dict(filter_attrs(attrs))
        cls = super(ChoicesMeta, cls).__new__(cls, name, bases, real_attrs)
        cls._dict = {v[0]: v[1] for k, v in attrs.items() if not k.startswith("__") and isinstance(v, tuple)}
        cls._items = list(cls._dict.items())
        cls._values = list(map(lambda x: x[1], cls._items))
        cls._keys = list(map(lambda x: x[0], cls._items))
        return cls

    def __contains__(self, val):
        return val in self.keys()

    def __getitem__(self, k):
        return self._dict[k]

    def keys(cls):
        return cls._keys

    def values(self):
        return self._values

    def items(self):
        return self._items

    def dict(self):
        return self._dict

    def get(self, key, default=None):
        return self._dict.get(key, default)

    def choices(self):
        return self.items()


class Choices(object, metaclass=ChoicesMeta):
    """ 自定义选择常量构造的用法
        In [18]: class FeedType(Choices):
            ...:     IMAGE = (1, '⽂文字图⽚片')
            ...:     VIDEO = (2, '⽂文字视频')
            ...:     ARTICLE = (3, '⽂文章')
            ...:

        In [19]: FeedType.IMAGE == 1
        Out[19]: True

        In [20]: FeedType.IMAGE_NAME == '文字图片'
        Out[20]: False

        In [21]: FeedType.IMAGE in FeedType
        Out[21]: True

        In [22]: FeedType.keys()
        Out[22]: [1, 2, 3]

        In [23]: FeedType.values()
        Out[23]: ['⽂文字图⽚片', '⽂文字视频', '⽂文章']

        In [24]: FeedType.items()
        Out[24]: [(1, '⽂文字图⽚片'), (2, '⽂文字视频'), (3, '⽂文章')]

        In [25]: FeedType.dict()
        Out[25]: {1: '⽂文字图⽚片', 2: '⽂文字视频', 3: '⽂文章'}

        In [26]: FeedType.choices()
        Out[26]: [(1, '⽂文字图⽚片'), (2, '⽂文字视频'), (3, '⽂文章')]

        In [27]: FeedType.get(FeedType.IMAGE)
        Out[27]: '⽂文字图⽚片'
    """


class ModelChoicesMeta(type):
    """数据库模型选择常量元类"""
    def __new__(cls, name, bases, data):
        cls = super(ModelChoicesMeta, cls).__new__(cls, name, bases, data)
        model_name, fields = data.get('data', [None, []])
        data_dict = []
        if model_name:
            try:
                ck_obj = model_name.objects.all().values_list(*fields)
                if ck_obj:
                    data_dict = list(ck_obj)
            except Exception as e:
                print(e)
        cls._dict = dict(data_dict)
        cls._items = list(cls._dict.items())
        cls._values = cls._dict.values()
        cls._keys = cls._dict.keys()
        return cls

    def __contains__(self, val):
        return val in self.keys()

    def __getitem__(self, k):
        return self._dict[k]

    def keys(cls):
        return cls._keys

    def values(self):
        return self._values

    def items(self):
        return self._items

    def dict(self):
        return self._dict

    def get(self, key, default=None):
        return self._dict.get(key, default)

    def choices(self):
        return self.items()


class ModelChoices(object, metaclass=ModelChoicesMeta):
    """
    使用数据库模型，指定字段，构造自定义选择常量的用法
    In [2]: class OAWorkFlowTypeChoices(ModelChoices):
       ...:
       ...:     data = (OAWorkFlowTypesModel, ('number', 'name'))
       ...:

    In [3]: OAWorkFlowTypeChoices.choices()
    Out[3]: [(1, 'KJ008'), (2, 'KJ021')]

    In [4]: OAWorkFlowTypeChoices.get(1)
    Out[4]: 'KJ008'

    In [5]: OAWorkFlowTypeChoices.get(2)
    Out[5]: 'KJ021'

    In [6]: OAWorkFlowTypeChoices.keys()
    Out[6]: dict_keys([1, 2])

    In [7]: OAWorkFlowTypeChoices.values()
    Out[7]: dict_values(['KJ008', 'KJ021'])

    In [8]: OAWorkFlowTypeChoices.items()
    Out[8]: [(1, 'KJ008'), (2, 'KJ021')]

    In [9]: OAWorkFlowTypeChoices.dict()
    Out[9]: {1: 'KJ008', 2: 'KJ021'}

    """


def set_custom_transaction_lock_data(new_data, old_data):
    """
    构造自定义事务锁数据更新
    :param new_data: `dict` 新数据库数据，数据库字段的对象数据需要反序列化，json.loads
    :param old_data: `dict` 旧的数据
    """
    # 防止数据调用被更新，做深度拷贝
    data = copy.deepcopy(new_data)
    _old_data = copy.deepcopy(old_data)
    # 遍历旧的数据，取出字段和字段值
    for o_k, o_v in _old_data.items():
        new_v = new_data.get(o_k)
        # 判断新的字段值是否为序列化对象
        if check_json_format(new_v):
            new_v = json.loads(new_v)
        # 判断旧的字段值是否为序列化对象
        if check_json_format(o_v):
            o_v = json.loads(o_v)

        # 新字段值是否为空
        if new_v:
            # 将新字段值，做深度拷贝
            _new_v = copy.deepcopy(new_v)
            if isinstance(new_v, list) and isinstance(o_v, list):
                new_vv = _new_v
                for ov in o_v:
                    if ov not in new_vv:
                        new_vv.append(ov)
                data[o_k] = json.dumps(new_vv)

            elif isinstance(new_v, dict) and isinstance(o_v, dict):
                new_vv = _new_v.update(o_v)
                data[o_k] = json.dumps(new_vv)

            elif new_v and o_v:
                if isinstance(o_v, (list, dict)):
                    o_v = json.dumps(o_v)
                data[o_k] = o_v
            else:
                pass
        else:
            if isinstance(o_v, (list, dict)):
                o_v = json.dumps(o_v)
            data[o_k] = o_v
    return data


def get_command_exec_result(command_str):
    """
    执行shell命令，且返回执行结果
    :param command_str: shell 命令字符串
    :return: (0, stdout)
    """
    return subprocess.getstatusoutput(command_str)


def check_and_mkdir_file_path_dir(file_path, is_dir=False):
    """
    检查传入文件的目录是否存在， 没有则创建
    :param file_path: 文件绝对路径
    :param is_dir: 传入路径是否只包含目录路径
    :return:
    """
    if not is_dir:
        file_dir_name = os.path.dirname(file_path)
    else:
        file_dir_name = file_path
    try:
        if not os.path.exists(file_dir_name):
            os.makedirs(file_dir_name)
        return True
    except Exception as e:
        return False


def write_text_file_add_lock(data, file_path, write_type="w"):
    """
    编写文本文件(加等待锁， 防止写入冲突)
    :param data: str
    :param file_path: 写入文件绝对路径, str
    :param write_type: 写类型, str, w或a, 即覆盖或追加
    :return:
    """
    try:
        check_and_mkdir_file_path_dir(file_path=file_path)
        with open(file_path, write_type) as f:
            fcntl.flock(f, fcntl.LOCK_EX)
            f.write(data)
            fcntl.flock(f, fcntl.LOCK_UN)
        return True
    except Exception as e:
        return False


def get_a_uuid():
    """
    获取一个UUID
    :return: str
    """
    uuid_obj = uuid.uuid4()
    uuid_str = str(uuid_obj)
    return uuid_str


def get_cur_host_mac_addr():
    """
    获取当前主机节点MAC地址
    :return:
    """
    addr = hex(uuid.getnode())[2:].upper()
    standard_addr = '0' * (12 - len(addr)) + addr
    standard_mac_addr = '-'.join(standard_addr[i:i + 2] for i in range(0, len(standard_addr), 2))
    return standard_mac_addr


def is_ip(ip_str):
    """检查字符串是否是IP"""
    ip_str = str(ip_str)
    p = re.compile('^((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$')
    if p.match(ip_str):
        return True
    else:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020-05-01 19:04
# @Author  : Jieay
# @File    : members_admin.py
from django.contrib import admin
from app.models.user.members import Members


@admin.register(Members)
class MembersAdmin(admin.ModelAdmin):
    list_display = (
        'id', 'username', 'name', 'is_staff', 'is_active', 'email', 'access_key', 'secret_key', 'ctime', 'mtime'
    )
    list_display_links = ('username',)
    list_filter = ('is_staff', 'is_active')
    search_fields = ('username', 'name')
    ordering = ('-id',)
    list_per_page = 10
